import { ApplicationRef, Component, ComponentFactoryResolver, Directive, ElementRef, EventEmitter, Injectable, Injector, Input, NgModule, Output, ReflectiveInjector, Renderer2, RendererFactory2, TemplateRef, ViewChild, ViewContainerRef } from '@angular/core';
import { BehaviorSubject } from 'rxjs/BehaviorSubject';
import 'rxjs/add/operator/filter';
import { CommonModule } from '@angular/common';

let NzbModalStatus = {};
NzbModalStatus.uninitialized = 0;
NzbModalStatus.initialized = 1;
NzbModalStatus.showing = 2;
NzbModalStatus.shown = 3;
NzbModalStatus.hiding = 4;
NzbModalStatus.hidden = 5;
NzbModalStatus[NzbModalStatus.uninitialized] = "uninitialized";
NzbModalStatus[NzbModalStatus.initialized] = "initialized";
NzbModalStatus[NzbModalStatus.showing] = "showing";
NzbModalStatus[NzbModalStatus.shown] = "shown";
NzbModalStatus[NzbModalStatus.hiding] = "hiding";
NzbModalStatus[NzbModalStatus.hidden] = "hidden";

class NzbModalComponent {
    constructor() {
        this.initialized = new EventEmitter();
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        setTimeout(() => {
            this.initialized.next();
        });
    }
}
NzbModalComponent.decorators = [
    { type: Component, args: [{
                selector: 'nzb-modal',
                template: `
		<div #modalElement tabindex="-1" role="dialog">
			<div #dialogElement role="document">
				<div class="modal-content">
					<ng-content></ng-content>
				</div>
			</div>
		</div>`
            },] },
];
/**
 * @nocollapse
 */
NzbModalComponent.ctorParameters = () => [];
NzbModalComponent.propDecorators = {
    'modalElement': [{ type: ViewChild, args: ['modalElement',] },],
    'dialogElement': [{ type: ViewChild, args: ['dialogElement',] },],
    'initialized': [{ type: Output },],
};

class NzbModalOptions {
    constructor() {
        this.backdrop = true;
        this.keyboard = true;
        this.focus = true;
        this.animation = true;
        this.size = null;
    }
}
NzbModalOptions.decorators = [
    { type: Injectable },
];
/**
 * @nocollapse
 */
NzbModalOptions.ctorParameters = () => [];

class NzbModalResult {
    constructor() {
        this.dismissed = true;
        this.data = null;
    }
}

class NzbDynamicContent {
    /**
     * @param {?} nodes
     * @param {?=} viewRef
     * @param {?=} componentRef
     */
    constructor(nodes, viewRef, componentRef) {
        this.nodes = nodes;
        this.viewRef = viewRef;
        this.componentRef = componentRef;
    }
}

class NzbDynamicContentComponent {
    /**
     * @param {?} appRef
     * @param {?} renderer
     * @param {?} element
     */
    constructor(appRef, renderer, element) {
        this.appRef = appRef;
        this.renderer = renderer;
        this.element = element;
        this.textNode = null;
    }
    /**
     * @param {?} content
     * @param {?=} context
     * @return {?}
     */
    updateContent(content, context) {
        this.container.clear();
        if (this.textNode) {
            this.renderer.removeChild(this.element.nativeElement, this.textNode);
            this.textNode = null;
        }
        if (content instanceof TemplateRef) {
            const /** @type {?} */ viewRef = content.createEmbeddedView(context);
            this.container.insert(viewRef);
        }
        else {
            if (typeof content == 'string') {
                this.textNode = this.renderer.createText(content);
                this.renderer.appendChild(this.element.nativeElement, this.textNode);
            }
        }
    }
}
NzbDynamicContentComponent.decorators = [
    { type: Component, args: [{
                selector: 'nzb-dynamic-content',
                template: `<ng-template #container></ng-template>`
            },] },
];
/**
 * @nocollapse
 */
NzbDynamicContentComponent.ctorParameters = () => [
    { type: ApplicationRef, },
    { type: Renderer2, },
    { type: ElementRef, },
];
NzbDynamicContentComponent.propDecorators = {
    'container': [{ type: ViewChild, args: ['container', { read: ViewContainerRef },] },],
};

/**
 * NzbDynamicContentFactoryService class.
 */
class NzbDynamicContentService {
    /**
     * @param {?} injector
     * @param {?} appRef
     * @param {?} cfr
     * @param {?} rendererFactory
     */
    constructor(injector, appRef, cfr, rendererFactory) {
        this.injector = injector;
        this.appRef = appRef;
        this.cfr = cfr;
        this.rendererFactory = rendererFactory;
        this.renderer = rendererFactory.createRenderer(null, null);
    }
    /**
     * Creates a dynamic content entry.
     * @param {?} content
     * @param {?} templateContext
     * @param {?} contentComponenInjector
     * @param {?} allowHTML
     * @return {?}
     */
    createDynamicContent(content, templateContext, contentComponenInjector, allowHTML) {
        if (!content) {
            return new NzbDynamicContent([]);
        }
        if (content instanceof TemplateRef) {
            const /** @type {?} */ viewRef = content.createEmbeddedView(templateContext);
            return new NzbDynamicContent([viewRef.rootNodes], viewRef);
        }
        if (typeof content === 'string') {
            let /** @type {?} */ el;
            if (allowHTML) {
                el = this.renderer.createElement('div');
                el.innerHTML = content;
            }
            else {
                el = this.renderer.createText(content);
            }
            return new NzbDynamicContent([[el]]);
        }
        try {
            const /** @type {?} */ componentFactory = this.cfr.resolveComponentFactory(content);
            const /** @type {?} */ componentRef = componentFactory.create(contentComponenInjector);
            return new NzbDynamicContent([[componentRef.location.nativeElement]], componentRef.hostView, componentRef);
        }
        catch (e) {
            return new NzbDynamicContent([]);
        }
    }
    /**
     * @param {?} dynamicContent
     * @return {?}
     */
    attach(dynamicContent) {
        const /** @type {?} */ factory = this.cfr.resolveComponentFactory(NzbDynamicContentComponent);
        const /** @type {?} */ componentRef = factory.create(this.injector, dynamicContent.nodes);
        if (dynamicContent.viewRef) {
            this.appRef.attachView(dynamicContent.viewRef);
        }
        this.appRef.attachView(componentRef.hostView);
        this.renderer.appendChild(document.body, componentRef.location.nativeElement);
        return componentRef;
    }
}
NzbDynamicContentService.decorators = [
    { type: Injectable },
];
/**
 * @nocollapse
 */
NzbDynamicContentService.ctorParameters = () => [
    { type: Injector, },
    { type: ApplicationRef, },
    { type: ComponentFactoryResolver, },
    { type: RendererFactory2, },
];

/**
 * NzbService class.
 */
class NzbService {
    /**
     * @param {?} cfr
     * @param {?} injector
     * @param {?} appRef
     * @param {?} rendererFactory
     * @param {?} contentService
     * @param {?} defaultModalOptions
     */
    constructor(cfr, injector, appRef, rendererFactory, contentService, defaultModalOptions) {
        this.cfr = cfr;
        this.injector = injector;
        this.appRef = appRef;
        this.rendererFactory = rendererFactory;
        this.contentService = contentService;
        this.defaultModalOptions = defaultModalOptions;
        this.elementCounter = 0;
        this.renderer = this.rendererFactory.createRenderer(null, null);
        this.elementCounter = 0;
    }
    /**
     * @param {?} prefix
     * @return {?}
     */
    getUniqueId(prefix) {
        prefix = prefix || 'nzb-generated-';
        this.elementCounter++;
        return prefix + this.elementCounter.toString();
    }
    /**
     * @return {?}
     */
    createModal() {
        return new NzbModal(this);
    }
}
NzbService.decorators = [
    { type: Injectable },
];
/**
 * @nocollapse
 */
NzbService.ctorParameters = () => [
    { type: ComponentFactoryResolver, },
    { type: Injector, },
    { type: ApplicationRef, },
    { type: RendererFactory2, },
    { type: NzbDynamicContentService, },
    { type: NzbModalOptions, },
];

class NzbModal {
    /**
     * Don't use this constructor directly. Use NzbService.createModal instead.
     * @see 	NzbService
     * @param {?} nzbService
     */
    constructor(nzbService) {
        this.nzbService = nzbService;
        this.ariaLabelledById = null;
        this.renderer = nzbService.renderer;
        this.defaultModalOptions = nzbService.defaultModalOptions;
        this.cfr = nzbService.cfr;
        this.injector = nzbService.injector;
        this.appRef = nzbService.appRef;
        this.contentService = nzbService.contentService;
        this.onShow = new EventEmitter();
        this.onShown = new EventEmitter();
        this.onHide = new EventEmitter();
        this.onHidden = new EventEmitter();
        this.status = new BehaviorSubject(NzbModalStatus.uninitialized);
        this.result = new BehaviorSubject(null);
    }
    /**
     * Shows the modal.
     * @param {?} content
     * @param {?=} options
     * @param {?=} ariaLabelledById
     * @return {?}
     */
    show(content, options, ariaLabelledById) {
        this._show(content, options, ariaLabelledById);
    }
    /**
     * Use this method to close the modal when the user "sucessfully" completes the modal.
     * The modal's `result.dismissed` will be false and `result.data` will be whatever you pass.
     * @param {?=} data
     * @return {?}
     */
    close(data) {
        const /** @type {?} */ result = new NzbModalResult();
        result.dismissed = false;
        result.data = data;
        this.result.next(result);
        this.hide();
    }
    /**
     * Use this method to indicate the user has "cancelled out" of the modal.
     * The modal's `result.dismissed` will be true and `result.data` will be null.
     * @return {?}
     */
    dismiss() {
        const /** @type {?} */ result = new NzbModalResult();
        result.dismissed = true;
        result.data = null;
        this.result.next(result);
        this.hide();
    }
    /**
     * A promise that resolves when the modal has been initialized but not yet shown.
     * @return {?}
     */
    initialized() {
        const /** @type {?} */ p = new Promise((resolve) => {
            const /** @type {?} */ sub = this.status
                .filter(val => val === NzbModalStatus.initialized).subscribe(val => {
                sub.unsubscribe();
                resolve();
            });
        });
        return p;
    }
    /**
     * A promise that resolves once when the modal is opened, i.e. after all the Bootstrap animations complete.
     * @return {?}
     */
    opened() {
        const /** @type {?} */ p = new Promise((resolve) => {
            const /** @type {?} */ sub = this.status
                .filter(val => val === NzbModalStatus.shown).subscribe(val => {
                sub.unsubscribe();
                resolve();
            });
        });
        return p;
    }
    /**
     * A promise that resolves once when the modal is hidden, i.e. after all the Bootstrap animations complete.
     * @return {?}
     */
    closed() {
        const /** @type {?} */ p = new Promise((resolve) => {
            const /** @type {?} */ sub = this.status
                .filter(val => val === NzbModalStatus.hidden).subscribe(val => {
                let /** @type {?} */ result = this.result.value;
                if (!this.result.value) {
                    this.result.next(new NzbModalResult());
                }
                sub.unsubscribe();
                resolve(this.result.value);
            });
        });
        return p;
    }
    /**
     * Use this method in ngOnDestroy to ensure the modal is hidden gracefully
     * when the component that created it is destroyed, e.g. on a route change.
     * @return {?}
     */
    destroy() {
        let /** @type {?} */ status = this.status.value;
        if (status === NzbModalStatus.shown || status === NzbModalStatus.showing || status === NzbModalStatus.initialized) {
            this.dismiss();
        }
        else {
            this.remove();
        }
    }
    /**
     * If you've opened the modal with a component type as content,
     * this will return the underlying ComponentRef.
     * @return {?}
     */
    getContentComponentRef() {
        if (this.dynamicContent && this.dynamicContent.componentRef) {
            return this.dynamicContent.componentRef;
        }
        return null;
    }
    /**
     * Get an observable of the modal status
     * @return {?}
     */
    getStatusObservable() {
        return this.status.asObservable();
    }
    /**
     * Get an observable of the modal result
     * @return {?}
     */
    getResultObservable() {
        return this.result.asObservable();
    }
    /**
     * @param {?} content
     * @param {?=} options
     * @param {?=} ariaLabelledById
     * @return {?}
     */
    _show(content, options, ariaLabelledById) {
        const /** @type {?} */ factory = this.cfr.resolveComponentFactory(NzbModalComponent);
        this.ariaLabelledById = ariaLabelledById ? ariaLabelledById : null;
        this.result.next(null);
        this.options = this.initializeOptions(options);
        this.dynamicContent = this.getContent(content);
        this.modalComponentRef = factory.create(this.injector, this.dynamicContent.nodes);
        const /** @type {?} */ sub = this.modalComponentRef.instance.initialized.subscribe(() => {
            this.onModalComponentInitialized();
            sub.unsubscribe();
        });
        this.appRef.attachView(this.modalComponentRef.hostView);
        if (this.dynamicContent.viewRef) {
            this.appRef.attachView(this.dynamicContent.viewRef);
        }
        this.renderer.appendChild(document.body, this.modalComponentRef.location.nativeElement);
    }
    /**
     * @return {?}
     */
    onModalComponentInitialized() {
        if (!this.modalComponentRef) {
            return;
        }
        const /** @type {?} */ $el = jQuery(this.modalComponentRef.instance.modalElement.nativeElement);
        const /** @type {?} */ options = Object.assign({}, this.options, { show: true });
        this.setAriaHidden(true);
        this.setAriaLabelledBy();
        this.initializeClasses();
        this.initializeEvents();
        this.status.next(NzbModalStatus.initialized);
        setTimeout(() => {
            $el.modal(options);
        });
    }
    /**
     * @return {?}
     */
    hide() {
        if (this.modalComponentRef) {
            const /** @type {?} */ $el = jQuery(this.modalComponentRef.instance.modalElement.nativeElement);
            $el.modal('hide');
        }
    }
    /**
     * @return {?}
     */
    remove() {
        this.removeEvents();
        if (this.modalComponentRef && this.dynamicContent) {
            if (this.dynamicContent.viewRef) {
                this.appRef.detachView(this.dynamicContent.viewRef);
            }
            this.appRef.detachView(this.modalComponentRef.hostView);
            this.modalComponentRef.destroy();
            this.status.next(NzbModalStatus.uninitialized);
        }
        this.modalComponentRef = null;
        this.dynamicContent = null;
    }
    /**
     * @param {?=} content
     * @return {?}
     */
    getContent(content) {
        return this.contentService.createDynamicContent(content, { modal: this }, ReflectiveInjector.resolveAndCreate([{ provide: NzbModal, useValue: this }], this.injector), true);
    }
    /**
     * @param {?} raw
     * @return {?}
     */
    initializeOptions(raw) {
        const /** @type {?} */ injected = typeof this.defaultModalOptions === 'object' ? this.defaultModalOptions : {};
        const /** @type {?} */ options = new NzbModalOptions();
        raw = typeof raw === 'object' ? raw : {};
        if (typeof raw.backdrop === 'boolean' || 'static' === raw.backdrop) {
            options.backdrop = raw.backdrop;
        }
        else {
            if (typeof injected.backdrop === 'boolean' || 'static' === injected.backdrop) {
                options.backdrop = injected.backdrop;
            }
        }
        if (typeof raw.keyboard === 'boolean') {
            options.keyboard = raw.keyboard;
        }
        else {
            if (typeof injected.keyboard === 'boolean') {
                options.keyboard = injected.keyboard;
            }
        }
        if (typeof raw.focus === 'boolean') {
            options.focus = raw.focus;
        }
        else {
            if (typeof injected.focus === 'boolean') {
                options.focus = injected.focus;
            }
        }
        if (typeof raw.animation === 'boolean') {
            options.animation = raw.animation;
        }
        else {
            if (typeof injected.animation === 'boolean') {
                options.animation = injected.animation;
            }
        }
        if (['sm', 'lg'].indexOf(raw.size) > -1) {
            options.size = raw.size;
        }
        else {
            if (['sm', 'lg'].indexOf(injected.size) > -1) {
                options.size = injected.size;
            }
        }
        return options;
    }
    /**
     * @return {?}
     */
    setAriaLabelledBy() {
        let /** @type {?} */ labelId;
        if (!this.modalComponentRef) {
            return;
        }
        const /** @type {?} */ el = this.modalComponentRef.instance.modalElement.nativeElement;
        const /** @type {?} */ $el = jQuery(el);
        if (typeof this.ariaLabelledById === 'string' && this.ariaLabelledById.trim().length > 0) {
            labelId = this.ariaLabelledById.trim();
        }
        else {
            labelId = this.nzbService.getUniqueId('nzb-modal-label-');
        }
        let /** @type {?} */ $label = jQuery('#' + labelId, $el);
        if ($label.length > 0) {
            this.renderer.setAttribute(el, 'aria-labelledby', labelId);
            return;
        }
        $label = jQuery('.modal-title', $el).first();
        if ($label.length > 0) {
            if ($label.attr('id') && $label.attr('id').length > 0) {
                labelId = $label.attr('id');
            }
            else {
                this.renderer.setAttribute($label.get(0), 'id', labelId);
            }
            this.renderer.setAttribute(el, 'aria-labelledby', labelId);
        }
    }
    /**
     * @param {?} hidden
     * @return {?}
     */
    setAriaHidden(hidden) {
        if (!this.modalComponentRef) {
            return;
        }
        const /** @type {?} */ el = this.modalComponentRef.instance.modalElement.nativeElement;
        this.renderer.setAttribute(el, 'aria-hidden', hidden ? 'true' : 'false');
    }
    /**
     * @return {?}
     */
    initializeClasses() {
        if (!this.modalComponentRef) {
            return;
        }
        const /** @type {?} */ modalEl = this.modalComponentRef.instance.modalElement.nativeElement;
        const /** @type {?} */ dialogEl = this.modalComponentRef.instance.dialogElement.nativeElement;
        this.renderer.addClass(modalEl, 'modal');
        if (this.options.animation) {
            this.renderer.addClass(modalEl, 'fade');
        }
        this.renderer.addClass(dialogEl, 'modal-dialog');
        if (this.options.size) {
            this.renderer.addClass(dialogEl, 'modal-' + this.options.size);
        }
    }
    /**
     * @return {?}
     */
    initializeEvents() {
        if (!this.modalComponentRef) {
            return;
        }
        const /** @type {?} */ $el = jQuery(this.modalComponentRef.instance.modalElement.nativeElement);
        $el.on('show.bs.modal shown.bs.modal hide.bs.modal hidden.bs.modal', (event) => {
            switch (event.type) {
                case 'show':
                    this.setAriaHidden(false);
                    this.status.next(NzbModalStatus.showing);
                    this.onShow.emit(event);
                    break;
                case 'shown':
                    this.status.next(NzbModalStatus.shown);
                    this.onShown.emit(event);
                    break;
                case 'hide':
                    this.status.next(NzbModalStatus.hiding);
                    this.onHide.emit(event);
                    break;
                case 'hidden':
                    this.setAriaHidden(true);
                    this.status.next(NzbModalStatus.hidden);
                    this.onHidden.emit(event);
                    this.remove();
                    break;
            }
        });
    }
    /**
     * @return {?}
     */
    removeEvents() {
        if (!this.modalComponentRef) {
            return;
        }
        const /** @type {?} */ $el = jQuery(this.modalComponentRef.instance.modalElement.nativeElement);
        $el.off('show.bs.modal shown.bs.modal hide.bs.modal hidden.bs.modal');
    }
}
NzbModal.decorators = [
    { type: Injectable },
];
/**
 * @nocollapse
 */
NzbModal.ctorParameters = () => [
    { type: NzbService, },
];

class NzbPopupOptions {
    constructor() {
        this.animation = false;
        this.container = false;
        this.delay = 0;
        this.html = false;
        this.placement = 'top';
        this.constraints = [];
        this.offset = '0 0';
        this.title = '';
    }
    /**
     * @param {?=} raw
     * @return {?}
     */
    normalize(raw) {
        raw = raw || {};
        if ('boolean' === typeof raw.animation) {
            this.animation = raw.animation;
        }
        if ('string' === typeof raw.container || false === raw.container) {
            this.container = raw.container;
        }
        if ('number' === typeof raw.delay || 'object' === typeof raw.delay) {
            this.delay = raw.delay;
        }
        if ('boolean' === typeof raw.html) {
            this.html = raw.html;
        }
        if (raw.placement) {
            this.placement = raw.placement;
        }
        if ('string' === typeof raw.template) {
            this.template = raw.template;
        }
        if ('string' === typeof raw.trigger) {
            this.trigger = raw.trigger;
        }
        if (raw.constraints) {
            this.constraints = raw.constraints;
        }
        if ('string' === typeof raw.offset) {
            this.offset = raw.offset;
        }
        if ('string' === typeof raw.title || raw.title instanceof TemplateRef) {
            this.title = raw.title;
        }
    }
}

class NzbPopoverOptions extends NzbPopupOptions {
    constructor() {
        super(...arguments);
        this.content = '';
        this.template = '<div class="popover" role="tooltip"><div class="popover-arrow"></div><h3 class="popover-title"></h3><div class="popover-content"></div></div>';
        this.trigger = 'click';
    }
    /**
     * @param {?=} raw
     * @return {?}
     */
    normalize(raw) {
        super.normalize(raw);
        raw = raw || {};
        if ('string' === typeof raw.content || raw.content instanceof TemplateRef) {
            this.content = raw.content;
        }
    }
}
NzbPopoverOptions.decorators = [
    { type: Injectable },
];
/**
 * @nocollapse
 */
NzbPopoverOptions.ctorParameters = () => [];

class NzbTooltipOptions extends NzbPopupOptions {
    constructor() {
        super(...arguments);
        this.template = '<div class="tooltip" role="tooltip"><div class="tooltip-arrow"></div><div class="tooltip-inner"></div></div>';
        this.trigger = 'click';
    }
    /**
     * @param {?=} raw
     * @return {?}
     */
    normalize(raw) {
        super.normalize(raw);
    }
}
NzbTooltipOptions.decorators = [
    { type: Injectable },
];
/**
 * @nocollapse
 */
NzbTooltipOptions.ctorParameters = () => [];

let NzbPopupStatus = {};
NzbPopupStatus.uninitialized = 0;
NzbPopupStatus.initialized = 1;
NzbPopupStatus.showing = 2;
NzbPopupStatus.shown = 3;
NzbPopupStatus.hiding = 4;
NzbPopupStatus.hidden = 5;
NzbPopupStatus[NzbPopupStatus.uninitialized] = "uninitialized";
NzbPopupStatus[NzbPopupStatus.initialized] = "initialized";
NzbPopupStatus[NzbPopupStatus.showing] = "showing";
NzbPopupStatus[NzbPopupStatus.shown] = "shown";
NzbPopupStatus[NzbPopupStatus.hiding] = "hiding";
NzbPopupStatus[NzbPopupStatus.hidden] = "hidden";

/**
 * @abstract
 */
class NzbPopup {
    /**
     * @param {?} bootstrapComponentName
     * @param {?} cfr
     * @param {?} injector
     * @param {?} appRef
     * @param {?} renderer
     * @param {?} element
     * @param {?} injectedOptions
     */
    constructor(bootstrapComponentName, cfr, injector, appRef, renderer, element, injectedOptions) {
        this.bootstrapComponentName = bootstrapComponentName;
        this.cfr = cfr;
        this.injector = injector;
        this.appRef = appRef;
        this.renderer = renderer;
        this.element = element;
        this.injectedOptions = injectedOptions;
        this.factory = this.cfr.resolveComponentFactory(NzbDynamicContentComponent);
        this.onShow = new EventEmitter();
        this.onShown = new EventEmitter();
        this.onHide = new EventEmitter();
        this.onHidden = new EventEmitter();
        this.statusSubject = new BehaviorSubject(NzbPopupStatus.uninitialized);
    }
    /**
     * @return {?}
     */
    get status() {
        return this.statusSubject.asObservable();
    }
    /**
     * @abstract
     * @return {?}
     */
    getContext() { }
    /**
     * @abstract
     * @return {?}
     */
    getInstanceOptions() { }
    /**
     * @return {?}
     */
    updateOptions() {
        let /** @type {?} */ instanceOptions = this.getInstanceOptions() || {};
        let /** @type {?} */ injectedOptions = this.injectedOptions || {};
        let /** @type {?} */ merged = Object.assign({}, injectedOptions, instanceOptions);
        this.options = this.bootstrapComponentName === 'tooltip' ?
            new NzbTooltipOptions() : new NzbPopoverOptions();
        this.options.normalize(merged);
    }
    /**
     * @return {?}
     */
    getBootstrapEvents() {
        const /** @type {?} */ phases = ['show', 'shown', 'hide', 'hidden'];
        return phases.map((phase) => {
            return phase + '.bs.' + this.bootstrapComponentName;
        });
    }
    /**
     * @param {?} target
     * @return {?}
     */
    initializeBootstrapEvents(target) {
        const /** @type {?} */ events = this.getBootstrapEvents();
        const /** @type {?} */ $el = jQuery(target.nativeElement);
        $el.on(events.join(' '), (event) => {
            switch (event.type) {
                case 'show':
                    this.setHideTriggers();
                    this.statusSubject.next(NzbPopupStatus.showing);
                    this.onShow.emit(event);
                    break;
                case 'shown':
                    this.statusSubject.next(NzbPopupStatus.shown);
                    this.onShown.emit(event);
                    break;
                case 'hide':
                    this.statusSubject.next(NzbPopupStatus.hiding);
                    this.onHide.emit(event);
                    break;
                case 'hidden':
                    this.statusSubject.next(NzbPopupStatus.hidden);
                    this.onHidden.emit(event);
                    break;
            }
        });
    }
    /**
     * @param {?} target
     * @return {?}
     */
    destroyBootstrapEvents(target) {
        const /** @type {?} */ events = this.getBootstrapEvents();
        const /** @type {?} */ $el = jQuery(target.nativeElement);
        $el.off(events.join(' '));
    }
    /**
     * @return {?}
     */
    initializeShowTriggers() {
        let /** @type {?} */ f;
        this.showTriggerUnlisteners = [];
        if (!this.options.trigger) {
            return null;
        }
        if (this.options.trigger !== 'clickOutside') {
            return this.options.trigger;
        }
        f = this.renderer.listen(this.element.nativeElement, 'click', () => {
            const /** @type {?} */ shown = this.statusSubject.value === NzbPopupStatus.shown;
            const /** @type {?} */ hidden = this.statusSubject.value === NzbPopupStatus.hidden;
            if (shown) {
                return this.hide();
            }
            if (hidden) {
                return this.show();
            }
        });
        this.showTriggerUnlisteners.push(f);
        return 'manual';
    }
    /**
     * @return {?}
     */
    setHideTriggers() {
        let /** @type {?} */ f;
        this.hideTriggerUnlisteners = [];
        if (!this.options.trigger) {
            return;
        }
        if (this.options.trigger !== 'clickOutside') {
            return;
        }
        const /** @type {?} */ maybeHide = (event) => {
            const /** @type {?} */ $el = jQuery(this.element.nativeElement);
            const /** @type {?} */ shown = this.statusSubject.value === NzbPopupStatus.shown;
            if (!shown) {
                return;
            }
            const /** @type {?} */ bsPopup = $el.data('bs.' + this.bootstrapComponentName);
            if (!bsPopup) {
                return;
            }
            const /** @type {?} */ tip = bsPopup.tip;
            const /** @type {?} */ isIn = jQuery.contains(tip, event.target);
            if (isIn) {
                return;
            }
            this.hideTriggerUnlisteners.forEach((f) => {
                f.call();
            });
            this.hideTriggerUnlisteners = [];
            this.hide();
        };
        f = this.renderer.listen(document.body, 'click', maybeHide);
        this.hideTriggerUnlisteners.push(f);
        f = this.renderer.listen(document.body, 'focusin', maybeHide);
        this.hideTriggerUnlisteners.push(f);
    }
    /**
     * @return {?}
     */
    updateComponents() {
        setTimeout(() => {
            const /** @type {?} */ context = this.getContext();
            if (this.titleComponentRef) {
                this.titleComponentRef.instance.updateContent(this.options.title, context);
            }
            if (this.contentComponentRef && this.options instanceof NzbPopoverOptions) {
                this.contentComponentRef.instance.updateContent(this.options.content, context);
            }
        });
    }
    /**
     * @return {?}
     */
    createDynamicContentComponent() {
        const /** @type {?} */ componentRef = this.factory.create(this.injector);
        this.appRef.attachView(componentRef.hostView);
        return componentRef;
    }
    /**
     * @return {?}
     */
    initTitleComponent() {
        this.titleComponentRef = this.createDynamicContentComponent();
    }
    /**
     * @return {?}
     */
    initContentComponent() {
        this.contentComponentRef = this.createDynamicContentComponent();
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        setTimeout(() => {
            this.updateOptions();
            const /** @type {?} */ $el = jQuery(this.element.nativeElement);
            const /** @type {?} */ attr = this.element.nativeElement.attributes.title;
            const /** @type {?} */ originalTitle = attr ? attr.value : '';
            const /** @type {?} */ trigger = this.initializeShowTriggers();
            let /** @type {?} */ options;
            if (originalTitle.length > 0) {
                if (!this.options.title) {
                    this.options.title = originalTitle;
                }
            }
            this.renderer.setAttribute(this.element.nativeElement, 'data-title-popup', originalTitle);
            this.renderer.removeAttribute(this.element.nativeElement, 'title');
            this.initializeBootstrapEvents(this.element);
            this.initTitleComponent();
            if ('popover' === this.bootstrapComponentName) {
                this.initContentComponent();
            }
            this.updateComponents();
            options = {
                html: true,
                title: this.titleComponentRef.location.nativeElement,
                animation: this.options.animation,
                container: this.options.container,
                delay: this.options.delay,
                placement: this.options.placement,
                template: this.options.template,
                constraints: this.options.constraints,
                offset: this.options.offset
            };
            if (trigger) {
                options.trigger = trigger;
            }
            if ('popover' === this.bootstrapComponentName) {
                options.content = this.contentComponentRef.location.nativeElement;
                $el.popover(options);
            }
            else {
                $el.tooltip(options);
            }
            this.statusSubject.next(NzbPopupStatus.hidden);
        });
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        const /** @type {?} */ $el = jQuery(this.element.nativeElement);
        this.destroyBootstrapEvents(this.element);
        this.hideTriggerUnlisteners.forEach((f) => {
            f.call();
        });
        this.hideTriggerUnlisteners = [];
        this.showTriggerUnlisteners.forEach((f) => {
            f.call();
        });
        this.showTriggerUnlisteners = [];
        if ('popover' === this.bootstrapComponentName) {
            $el.popover('dispose');
        }
        else {
            $el.tooltip('dispose');
        }
    }
    /**
     * @return {?}
     */
    ngOnChanges() {
        this.updateComponents();
    }
    /**
     * @return {?}
     */
    show() {
        const /** @type {?} */ $el = jQuery(this.element.nativeElement);
        if ('popover' === this.bootstrapComponentName) {
            $el.popover('show');
        }
        else {
            $el.tooltip('show');
        }
    }
    /**
     * @return {?}
     */
    hide() {
        const /** @type {?} */ $el = jQuery(this.element.nativeElement);
        if ('popover' === this.bootstrapComponentName) {
            $el.popover('hide');
        }
        else {
            $el.tooltip('hide');
        }
    }
    /**
     * A promise that resolves once when the popup is opened, i.e. after all the Bootstrap animations complete.
     * @return {?}
     */
    shown() {
        const /** @type {?} */ p = new Promise((resolve) => {
            const /** @type {?} */ sub = this.statusSubject
                .filter(val => val === NzbPopupStatus.shown).subscribe(val => {
                sub.unsubscribe();
                resolve();
            });
        });
        return p;
    }
    /**
     * A promise that resolves once when the popup is opened, i.e. after all the Bootstrap animations complete.
     * @return {?}
     */
    hidden() {
        const /** @type {?} */ p = new Promise((resolve) => {
            const /** @type {?} */ sub = this.statusSubject
                .filter(val => val === NzbPopupStatus.hidden).subscribe(val => {
                sub.unsubscribe();
                resolve();
            });
        });
        return p;
    }
}

class NzbTooltipDirective extends NzbPopup {
    /**
     * @param {?} element
     * @param {?} cfr
     * @param {?} injector
     * @param {?} appRef
     * @param {?} renderer
     * @param {?} injectedOptions
     */
    constructor(element, cfr, injector, appRef, renderer, injectedOptions) {
        super('tooltip', cfr, injector, appRef, renderer, element, injectedOptions);
    }
    /**
     * @return {?}
     */
    getContext() {
        return { popover: this };
    }
    /**
     * @return {?}
     */
    getInstanceOptions() {
        return this.nzbTooltip;
    }
}
NzbTooltipDirective.decorators = [
    { type: Directive, args: [{
                selector: '[nzbTooltip]',
                exportAs: 'nzbTooltip'
            },] },
];
/**
 * @nocollapse
 */
NzbTooltipDirective.ctorParameters = () => [
    { type: ElementRef, },
    { type: ComponentFactoryResolver, },
    { type: Injector, },
    { type: ApplicationRef, },
    { type: Renderer2, },
    { type: NzbTooltipOptions, },
];
NzbTooltipDirective.propDecorators = {
    'nzbTooltip': [{ type: Input },],
};

class NzbPopoverDirective extends NzbPopup {
    /**
     * @param {?} element
     * @param {?} cfr
     * @param {?} injector
     * @param {?} appRef
     * @param {?} renderer
     * @param {?} injectedOptions
     */
    constructor(element, cfr, injector, appRef, renderer, injectedOptions) {
        super('popover', cfr, injector, appRef, renderer, element, injectedOptions);
        this.name = 'MMnjhkjhsjkhs';
    }
    /**
     * @return {?}
     */
    getContext() {
        return { popover: this };
    }
    /**
     * @return {?}
     */
    getInstanceOptions() {
        return this.nzbPopover;
    }
}
NzbPopoverDirective.decorators = [
    { type: Directive, args: [{
                selector: '[nzbPopover]',
                exportAs: 'nzbPopover'
            },] },
];
/**
 * @nocollapse
 */
NzbPopoverDirective.ctorParameters = () => [
    { type: ElementRef, },
    { type: ComponentFactoryResolver, },
    { type: Injector, },
    { type: ApplicationRef, },
    { type: Renderer2, },
    { type: NzbPopoverOptions, },
];
NzbPopoverDirective.propDecorators = {
    'nzbPopover': [{ type: Input },],
};

class NzbModule {
}
NzbModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule
                ],
                declarations: [
                    NzbDynamicContentComponent,
                    NzbModalComponent,
                    NzbTooltipDirective,
                    NzbPopoverDirective
                ],
                exports: [
                    NzbDynamicContentComponent,
                    NzbModalComponent,
                    NzbTooltipDirective,
                    NzbPopoverDirective
                ],
                entryComponents: [
                    NzbDynamicContentComponent,
                    NzbModalComponent
                ],
                providers: [
                    NzbService,
                    NzbDynamicContentService,
                    NzbModalOptions,
                    NzbTooltipOptions,
                    NzbPopoverOptions
                ]
            },] },
];
/**
 * @nocollapse
 */
NzbModule.ctorParameters = () => [];

// Public classes.

/**
 * Angular library starter.
 * Build an Angular library compatible with AoT compilation & Tree shaking.
 * Written by Roberto Simonetti.
 * MIT license.
 * https://github.com/robisim74/nowzoo-angular-bootstrap-lite
 */
/**
 * Entry point for all public APIs of the package.
 */

/**
 * Generated bundle index. Do not edit.
 */

export { NzbModalStatus, NzbModal, NzbModalOptions, NzbModalResult, NzbModalComponent, NzbPopupOptions, NzbPopoverOptions, NzbTooltipOptions, NzbTooltipDirective, NzbPopoverDirective, NzbDynamicContent, NzbDynamicContentService, NzbDynamicContentComponent, NzbService, NzbModule, NzbPopup as ɵa };
//# sourceMappingURL=nowzoo-angular-bootstrap-lite.js.map
